From 97696d2c1c146fd7d3161f592d42cd7d73c6d5a8 Mon Sep 17 00:00:00 2001
From: qing zhao <qing.zhao@oracle.com>
Date: Thu, 23 Apr 2020 16:55:24 -0400
Subject: [PATCH] Fix PR94230

---
 gcc/c-family/c-indentation.c                          |  4 ++++
 gcc/common.opt                                        |  5 +++++
 gcc/doc/invoke.texi                                   | 19 +++++++++++++++++--
 .../gcc.dg/plugin/location-overflow-test-1.c          |  2 +-
 gcc/toplev.c                                          |  3 +++
 5 files changed, 30 insertions(+), 3 deletions(-)

diff --git a/gcc/c-family/c-indentation.c b/gcc/c-family/c-indentation.c
index f737555..e929030 100644
--- a/gcc/c-family/c-indentation.c
+++ b/gcc/c-family/c-indentation.c
@@ -67,6 +67,10 @@ get_visual_column (expanded_location exploc, location_t loc,
 		  "%<-Wmisleading-indentation%> is disabled from this point"
 		  " onwards, since column-tracking was disabled due to"
 		  " the size of the code/headers");
+	  if (!flag_large_source_files)
+	    inform (loc,
+		    "please add %<-flarge-source-files%> to invoke more" 
+		    " column-tracking for large source files");
 	}
       return false;
     }
diff --git a/gcc/common.opt b/gcc/common.opt
index 4368910..03fd4be 100644
--- a/gcc/common.opt
+++ b/gcc/common.opt
@@ -1597,6 +1597,11 @@ fkeep-gc-roots-live
 Common Undocumented Report Var(flag_keep_gc_roots_live) Optimization
 ; Always keep a pointer to a live memory block
 
+flarge-source-files
+Common Report Var(flag_large_source_files) Init(0)
+Improve GCC's ability to track column numbers in large source files,
+at the expense of slower compilation.
+
 floop-parallelize-all
 Common Report Var(flag_loop_parallelize_all) Optimization
 Mark all loops as parallel.
diff --git a/gcc/doc/invoke.texi b/gcc/doc/invoke.texi
index 96a9516..cd580ab 100644
--- a/gcc/doc/invoke.texi
+++ b/gcc/doc/invoke.texi
@@ -573,8 +573,8 @@ Objective-C and Objective-C++ Dialects}.
 -dD  -dI  -dM  -dN  -dU @gol
 -fdebug-cpp  -fdirectives-only  -fdollars-in-identifiers  @gol
 -fexec-charset=@var{charset}  -fextended-identifiers  @gol
--finput-charset=@var{charset}  -fmacro-prefix-map=@var{old}=@var{new}  @gol
--fmax-include-depth=@var{depth} @gol
+-finput-charset=@var{charset}  -flarge-source-files  @gol
+-fmacro-prefix-map=@var{old}=@var{new} -fmax-include-depth=@var{depth} @gol
 -fno-canonical-system-headers  -fpch-deps  -fpch-preprocess  @gol
 -fpreprocessed  -ftabstop=@var{width}  -ftrack-macro-expansion  @gol
 -fwide-exec-charset=@var{charset}  -fworking-directory @gol
@@ -14151,6 +14151,21 @@ This option may be useful in conjunction with the @option{-B} or
 perform additional processing of the program source between
 normal preprocessing and compilation.
 
+@item -flarge-source-files
+@opindex flarge-source-files
+Adjust GCC to expect large source files, at the expense of slower
+compilation and higher memory usage.
+
+Specifically, GCC normally tracks both column numbers and line numbers
+within source files and it normally prints both of these numbers in
+diagnostics.  However, once it has processed a certain number of source
+lines, it stops tracking column numbers and only tracks line numbers.
+This means that diagnostics for later lines do not include column numbers.
+It also means that options like @option{-Wmisleading-indent} cease to work
+at that point, although the compiler prints a note if this happens.
+Passing @option{-flarge-source-files} significantly increases the number
+of source lines that GCC can process before it stops tracking column.
+
 @end table
 
 @node Assembler Options
diff --git a/gcc/testsuite/gcc.dg/plugin/location-overflow-test-1.c b/gcc/testsuite/gcc.dg/plugin/location-overflow-test-1.c
index 1a80a66..fbc8211 100644
--- a/gcc/testsuite/gcc.dg/plugin/location-overflow-test-1.c
+++ b/gcc/testsuite/gcc.dg/plugin/location-overflow-test-1.c
@@ -13,7 +13,7 @@ int
 fn_1 (int flag)
 {
   int x = 4, y = 5;
-  if (flag) x = 3; y = 2; /* { dg-message "-:disabled from this point" } */
+  if (flag) x = 3; y = 2; /* { dg-message "-:disabled from this point" "add '-flarge-source-files'" } */
   return x * y;
 }
 
diff --git a/gcc/toplev.c b/gcc/toplev.c
index 4c8be50..b3230b7 100644
--- a/gcc/toplev.c
+++ b/gcc/toplev.c
@@ -1854,6 +1854,9 @@ process_options (void)
     hash_table_sanitize_eq_limit
       = param_hash_table_verification_limit;
 
+  if (flag_large_source_files)
+    line_table->default_range_bits = 0;
+
   /* Please don't change global_options after this point, those changes won't
      be reflected in optimization_{default,current}_node.  */
 }
-- 
1.8.3.1

